# Copyright (C) 2023 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import asyncio

import yaml
from headless.core import httpx

from molano.lib.sku import v1


async def main():
    fn: str = v1.__file__.replace('.py', '.yaml')
    async with httpx.Client(base_url="") as client:
        with open(fn, 'w') as f:
            f.write(
                '---\n' +
                yaml.safe_dump( # type: ignore
                    [
                        *[row async for row in v1.parse_ipads(client)],
                        *[row async for row in v1.parse_iphones(client)],
                    ],
                    indent=2,
                    default_flow_style=False
                )
            )


if __name__ == '__main__':
    asyncio.run(main())


# Copyright (C) 2023 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# type: ignore
import asyncio
import collections
import copy
import hashlib
import re
import sys
from typing import Any

import yaml
from headless.core import httpx

from molano.lib import appleparts
from molano.lib import spot
from molano.lib.sku import v1
from molano.lib.sku import v2
from molano.lib.sku import extract


_generations: dict[int, str] = collections.defaultdict(lambda: '{n}th', {
    1: '{n}st',
    2: '{n}nd',
    3: '{n}rd',
})


APPLE_GRADES: list[tuple[str, str]] = [
    ('0', 'A+'),
    ('1', 'A'),
    ('2', 'B+'),
    ('3', 'B'),
    ('4', 'C+'),
    ('5', 'C')
]

def create_battery(params: dict[str, Any]) -> dict[str, Any]:
    params = copy.deepcopy(params)
    base = params['base']
    if base.startswith('IPHSE'):
        base = f"{base}{params['year']}"
    search_keys: list[str] = []

    h = hashlib.sha1()
    h.update(str.encode('apple', 'utf-8'))
    h.update(str.encode('apple', 'utf-8'))
    h.update(str.encode(base, 'utf-8'))

    return {
        'kind': 'Part',
        'catalog_name': f"{params['series']} Battery + Adhesive Tape",
        'name': f"{params['series']} Battery + Adhesive Tape",
        'namespaces': ['catalog.buymolano.com'],
        'product_type': 'FINISHED_GOOD',
        'search_keys': [
            h.hexdigest(),
            f"BATT{base}"
        ]
    }

def create_msin_name(product: dict[str, Any]) -> str | None:
    msin_name = None
    if product['kind'] == 'iPad':
        msin_name = f"{product['model']}"
        if product['model'] == 'iPad Pro':
            msin_name += f" {product['screen_size']}\""
        msin_name += f" {product['storage']}"
        msin_name += ' Wi-Fi' if not product['cellular'] else ' Wi-Fi + Cellular'
        msin_name += f" {product['color']}"
        generation = format_generation(product['generation']) + ' generation'
        if product['restricted'] == 'CN':
            generation += ', China'
        if product['restricted'] == 'JP':
            generation += ', Japan'
        msin_name = f'{msin_name} ({generation})'
    elif product['kind'] == 'iPhone':
        msin_name = "{model} {storage} {color}".format(**product)
        if product['restricted'] == 'CN':
            msin_name = f'{msin_name} (China)'
        if product['restricted'] == 'JP':
            msin_name = f'{msin_name} (Japan)'
    return msin_name


def create_variant_name(
    product: dict[str, Any],
    variant: dict[str, Any],
    color: str
) -> str:
    name = f"{product['name']} {color}"
    if product['kind'] == 'iPhone':
        name = '{model} {storage} {color}'.format(
            color=color,
            **product
        )
        if product['restricted'] == 'CN':
            name = f'{name} China'
        if product['restricted'] == 'JP':
            name = f'{name} Japan'
    return name


def create_bulk_name(params: dict[str, Any]) -> str | None:
    if params['kind'] != 'iPhone':
        return None
    memory = re.sub('^([0-9]+)(GB|TB)$', r'\1 \2', params['memory'])
    if params.get('series', '').startswith('iPhone SE'):
        name = f"iPhone SE ({params['year']})"
        if str.endswith(params.get('tag') or '', 'jp'):
            name = f"iPhone SE ({params['year']}, Japan)"
        elif str.endswith(params.get('tag') or '', 'cn'):
            name = f"iPhone SE ({params['year']}, China)"
        name += f" {memory}"
    else:
        name = f"{params['series']} {memory}"
        if str.endswith(params.get('tag') or '', 'jp'):
            name = f"{name} (Japan)"
        elif str.endswith(params.get('tag') or '', 'cn'):
            name = f"{name} (China)"
    return f'Bulk {name}'

def create_product_name(params: dict[str, Any]) -> str:
    memory = re.sub('^([0-9]+)(GB|TB)$', r'\1 \2', params['memory'])
    if params.get('series', '').startswith('iPhone SE'):
        name = f"iPhone SE ({params['year']})"
        if str.endswith(params.get('tag') or '', 'jp'):
            name = f"iPhone SE ({params['year']}, Japan)"
        elif str.endswith(params.get('tag') or '', 'cn'):
            name = f"iPhone SE ({params['year']}, China)"
        name += f" {memory}"
    elif params['kind'] == 'iPhone':
        name = f"{params['series']} {memory}"
        if str.endswith(params.get('tag') or '', 'jp'):
            name = f"{name} (Japan)"
        elif str.endswith(params.get('tag') or '', 'cn'):
            name = f"{name} (China)"
    elif params['kind'] == 'iPad':
        name = f"{params['device']}"
        if params.get('screen_size') and params.get('series') not in {'iPad', 'iPad mini'} and params.get('device') != 'iPad':
            name = f"{name} {params['screen_size']}\""
        if str.endswith(params.get('tag') or '', 'jp'):
            name = f"{name} ({params['year']}, Japan)"
        elif str.endswith(params.get('tag') or '', 'cn'):
            name = f"{name} ({params['year']}, China)"
        else:
            name = f"{name} ({params['year']})"
        name += f" {memory}"
        name += f" {'Cellular' if params.get('cellular') else 'Wi-Fi'}"
    else:
        raise NotImplementedError
    return name



def format_generation(n: int) -> str:
    return _generations[n].format(n=n)

def get_catalog_name(params: dict[str, Any]) -> str | None:
    name = None
    if params['kind'] == 'iPhone':
        name = f"{params['model']} {params['storage']} {params['color']} {params['grade_name']}"
    if params['kind'] == 'iPad':
        name = f"{params['model']} {params['storage']} ({params['year']})"
        if params.get('restricted') == 'CN':
            name = f"{params['model']} {params['storage']} ({params['year']}, China)"
        if params.get('restricted') == 'JP':
            name = f"{params['model']} {params['storage']} ({params['year']}, Japan) "
        if params['model'] == 'iPad Pro':
            name += f" {params['screen_inch']}\""
        name += " Wi-Fi" if not params.get('cellular') else " Wi-Fi + Cellular"
        name += f" {params['color']} {params['grade_name']}"
    return name


async def main():
    async with httpx.Client(base_url="", timeout=10) as client:
        parts = [
            #*(await appleparts.parse(client)),
            #*(await spot.parse_selfservicerepair(client))
        ]

        models = [
            #[row async for row in extract.parse_ipads(client, v2.APPLE_COLORS)],
            *[row async for row in extract.parse_iphones(client, v2.APPLE_COLORS)],
        ]

        # Create bulk
        series: dict[str, dict[str, Any]] = collections.defaultdict(lambda: {
            'features': [],
        })
        bulk: dict[tuple[str, str, str], dict[str, str | set[str]]] = collections.defaultdict(
            lambda: {
                'identifiers': [],
                'features': []
            }
        )
        batteries: dict[str, Any] = {}
        grades: dict[str, str] = {
            'A+'    : '0',
            'A'     : '1',
            'B+'    : '2',
            'B'     : '3',
            'C+'    : '4',
            'C'     : '5',
            'Any'   : None
        }
        for model in models:
            # Group products by series, market, storage and collect possible colors
            if model.get('year') < 2014:
                continue
            name = create_product_name(model)
            storage = {
                'kind': 'Generic',
                'category': 'primaryhd',
                'applicable': 'REQ',
                'label': re.sub('^([0-9]+)(GB|TB)$', r'\1 \2', model['memory'])
            }
            product = series[name]
            product['product_name'] = name
            if storage not in product['features']:
                product['features'].append(storage)

            kind_feature = {
                'kind': 'Generic',
                'category': 'apple.com/kind',
                'applicable': 'REQ',
                'label': model['kind']
            }
            if kind_feature not in product['features']:
                product['features'].append(kind_feature)

            model_feature = {
                'kind': 'Generic',
                'category': 'apple.com/model',
                'applicable': 'REQ',
                'label': model['series']
            }
            if model_feature not in product['features']:
                product['features'].append(model_feature)
            color_feature = {
                'kind': 'Generic',
                'category': 'apple.com/color',
                'applicable': 'SEL',
                'unique': True,
                'label': model['color']
            }
            if color_feature not in product['features']:
                product['features'].append(color_feature)
            restrict = None
            country = None
            if model.get('tag'):
                t = {
                    'CN': 'China',
                    'JP': 'Japan'
                }
                country = str.upper(model['tag'].split(':')[-1])
                restrict = {
                    'kind': 'Generic',
                    'category': 'apple.com/restriction',
                    'applicable': 'REQ',
                    'label': country
                }
                if restrict not in product['features']:
                    product['features'].insert(0, restrict)
            else:
                product['dropship'] = True
            for grade in grades.keys():
                ft = {
                    'kind': 'Generic',
                    'category': 'molano.nl/grade',
                    'applicable': 'SEL',
                    'label': grade
                }
                if ft not in product['features']:
                    product['features'].append(ft)

            bulk_name = create_bulk_name(model)
            if bulk_name:
                product = bulk[bulk_name]
                product['product_name'] = bulk_name
                product['bulk'] = True
                sku = f"{model['base']}{model['memory']}"
                if country:
                    sku = f"{sku}{country}"
                if sku not in product['identifiers']:
                    product['identifiers'].append(sku)
                if storage not in product['features']:
                    product['features'].append(storage)
                mixed = {
                    'kind': 'Generic',
                    'category': 'apple.com/color',
                    'applicable': 'SEL',
                    'label': 'Mixed'
                }
                if mixed not in product['features']:
                    product['features'].append(mixed)
                color_feature = {
                    'kind': 'Generic',
                    'category': 'apple.com/color',
                    'applicable': 'SEL',
                    'label': model['color']
                }
                if color_feature not in product['features']:
                    product['features'].append(color_feature)
                if kind_feature not in product['features']:
                    product['features'].append(kind_feature)
                if model_feature not in product['features']:
                    product['features'].append(model_feature)
                if restrict and restrict not in product['features']:
                    product['features'].insert(0, restrict)
                for grade in grades.keys():
                    ft = {
                        'kind': 'Generic',
                        'category': 'molano.nl/grade',
                        'applicable': 'SEL',
                        'label': grade
                    }
                    if ft not in product['features']:
                        product['features'].append(ft)
            continue
            #if model.get('kind') == 'iPhone':
            #    batteries[model['base']] = create_battery(model)
            cap = model.get('cellular')
            k = (
                model['series'],
                model['tag'] or '',
                model['memory'],
                model.get('cellular') or ''
            )
            product = bulk[k]
            product['base'] = model['base']
            product['cellular'] = model.get('cellular')
            product['product_type'] = 'RAW_MATERIAL'
            product['colors'].add(model['color'])
            product['firmware_versions'].add(model['identifier'])
            product['generation'] = model['generation']
            product['kind'] = model['kind']
            product['model'] = model['series']
            product['namespaces'] = []
            product['search_keys'] = []
            product['storage'] = model['memory']
            product['year'] = model['year']

            h = hashlib.sha1()
            h.update('apple'.encode('utf-8'))
            for x in k:
                if isinstance(x, bool):
                    x = 't' if x else 'f'
                h.update(str.encode(x.lower(), 'utf-8'))
            product['search_keys'].append(f'sha1:{h.hexdigest()}')
            if product['kind'] == 'iPhone':
                product['name'] = f"{model['series']} {model['memory']}"

            elif product['kind'] == 'iPad':
                product['model'] = model['device']
                n = f"{model['device']}"
                if model.get('screen_size') and model.get('series') not in {'iPad', 'iPad mini'} and model.get('device') != 'iPad':
                    n = f"{n} {model['screen_size']}\""
                n = f"{n} ({model['year']}) {model['memory']}"
                if cap:
                    n = f"{n} Wi-Fi + Cellular"
                else:
                    n = f"{n} Wi-Fi"
                product['name'] = n
                product['screen_inch'] = model['screen_inch']
                product['screen_size'] = model['screen_size']


            product['restricted'] = None
            if str.endswith(model.get('tag') or '', 'jp'):
                product['restricted'] = 'JP'
            if str.endswith(model.get('tag') or '', 'cn'):
                product['restricted'] = 'CN'

        items: list[Any] = [x[1] for x in sorted(batteries.items(), key=lambda x: x[0])]
        for k, product in bulk.items():
            continue
            product = copy.deepcopy(product)
            product['name'] = f"Bulk {product['name']}"
            product['colors'] = list(sorted(product['colors']))
            product['firmware_versions'] = list(sorted(product['firmware_versions']))

        # Create variants
        for k, product in bulk.items():
            continue
            product = copy.deepcopy(product)
            product['grade'] = None
            if product.get('kind') in {'iPhone', 'iPad'}:
                for color in product.pop('colors'):
                    assert len(product['search_keys']) == 1
                    h = hashlib.sha1()
                    h.update(str.encode(product['search_keys'][0], 'utf-8'))
                    h.update(str.encode(color, 'utf-8'))
                    variant = copy.deepcopy(product)
                    variant['search_keys'] = [f'sha1:{h.hexdigest()}']
                    variant['msin'] = True
                    variant['msin_key'] = variant['search_keys'][0]
                    variant['color'] = color
                    variant['firmware_versions'] = list(sorted(variant['firmware_versions']))
                    variant['name'] = create_variant_name(product, variant, color)
                    items.append(variant)

                    for grade, name in APPLE_GRADES:
                        assert len(variant['search_keys']) == 1
                        h = hashlib.sha1()
                        h.update(str.encode(variant['search_keys'][0], 'utf-8'))
                        h.update(str.encode(grade, 'utf-8'))
                        graded = {**copy.deepcopy(variant), 'msin': variant['search_keys'][0], 'msin_key': None}
                        graded['search_keys'] = [f'sha1:{h.hexdigest()}']
                        graded['name'] = f"{graded['name']} {name}"
                        graded['grade'] = grade
                        graded['grade_name'] = name
                        graded['product_type'] = 'FINISHED_GOOD'
                        graded['namespaces'].append('catalog.buymolano.com')
                        if graded['kind'] in {'iPhone', 'iPad'}:
                            graded['search_keys'].extend([
                                #v1.generate_sku(graded),
                                v2.generate_sku(graded)
                            ])
                        graded['catalog_name'] = get_catalog_name(graded)
                        items.append(graded)

                    h = hashlib.sha1()
                    h.update(str.encode(variant['search_keys'][0], 'utf-8'))
                    h.update(str.encode('6', 'utf-8'))
                    defunct =  {**copy.deepcopy(variant), 'msin': variant['search_keys'][0], 'msin_key': None}
                    defunct['search_keys'] = []
                    defunct['name'] = f"{variant['name']} (6 COSMETIC FAIL)"
                    defunct['grade'] = '6'
                    defunct['product_type'] = 'SUBASSEMBLY'
                    defunct['search_keys'].extend([
                        f'sha1:{h.hexdigest()}',
                        v2.generate_sku(defunct)
                    ])
                    items.append(defunct)

                    h = hashlib.sha1()
                    h.update(str.encode(variant['search_keys'][0], 'utf-8'))
                    h.update(str.encode('7', 'utf-8'))
                    functional =  {**copy.deepcopy(variant), 'msin': variant['search_keys'][0], 'msin_key': None}
                    functional['search_keys'] = []
                    functional['name'] = f"{variant['name']} (7 FUNCTIONAL FAIL)"
                    functional['grade'] = '7'
                    functional['product_type'] = 'SUBASSEMBLY'
                    functional['search_keys'].extend([
                        f'sha1:{h.hexdigest()}',
                        v2.generate_sku(functional)
                    ])
                    items.append(functional)

                    h = hashlib.sha1()
                    h.update(str.encode(variant['search_keys'][0], 'utf-8'))
                    h.update(str.encode('8', 'utf-8'))
                    bat84 =  {**copy.deepcopy(variant), 'msin': variant['search_keys'][0], 'msin_key': None}
                    bat84['search_keys'] = []
                    bat84['name'] = f"{variant['name']} (8 BATTERY 80-84)"
                    bat84['grade'] = '8'
                    bat84['product_type'] = 'SUBASSEMBLY'
                    bat84['search_keys'].extend([
                        f'sha1:{h.hexdigest()}',
                        v2.generate_sku(bat84)
                    ])
                    items.append(bat84)

                    h = hashlib.sha1()
                    h.update(str.encode(variant['search_keys'][0], 'utf-8'))
                    h.update(str.encode('9', 'utf-8'))
                    bat79 =  {**copy.deepcopy(variant), 'msin': variant['search_keys'][0], 'msin_key': None}
                    bat79['search_keys'] = []
                    bat79['name'] = f"{variant['name']} (9 BATTERY BELOW 80)"
                    bat79['grade'] = '9'
                    bat79['product_type'] = 'SUBASSEMBLY'
                    bat79['search_keys'].extend([
                        f'sha1:{h.hexdigest()}',
                        v2.generate_sku(bat79)
                    ])
                    items.append(bat79)
                    variant['msin_name'] = create_msin_name(variant)

            items.append(product)
        items = [
            *parts,
            *items,
            *sorted(bulk.values(), key=lambda x: x['product_name']),
            *sorted(series.values(), key=lambda x: x['product_name'])
        ]
        print(yaml.safe_dump(items, default_flow_style=False))
        print(f"Parsed {len(items)} products", file=sys.stderr)

if __name__ == '__main__':
    asyncio.run(main())


# Copyright (C) 2023 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import asyncio
import copy

from headless.core import httpx

from molano.lib.sku import v1
from molano.lib.sku import v2
from molano.lib.sku import generate
from molano.lib.sheets import Spreadsheet
from molano.lib.sheets import SpreadsheetModel


DEFAULT_GRADINGS: dict[str, str] = {
    'A+'    : '0',
    'A'     : '1',
    'B+'    : '2',
    'B'     : '3',
    'C+'    : '4',
    'C'     : '5',
    'D'     : '6'
}


class PhoneInfo(SpreadsheetModel):
    name: str
    model: str
    skuv1: str
    skuv2: str
    kind: str
    grading_level: str = 'ABC'
    grade: str
    body_grade: str = ""
    screen_grade: str = ""
    device: str
    memory: str
    year: str
    screen_size: str = ""
    parts_category: str = ""
    accessoires_category: str = ""
    vat: str = 'VAT'
    color: str


async def main():
    sheet = Spreadsheet(
        model=PhoneInfo,
        id='12fZ4JCiPyBwerZULjXZB40ecgAaHDy-HE_y19nrJLaM',
        sheet_id=1571318229,
        range='Apple Products'
    )
    sheet.clear()
    rows: list[PhoneInfo] = []
    async with httpx.Client(base_url="") as client:
        async for params in generate(client, using=v1, gradings=DEFAULT_GRADINGS, with_ungraded=False):
            if params.get('tag') in {'apple:model:exclusive:cn', 'apple:model:exclusive:jp'}:
                continue
            row = PhoneInfo.parse_obj({
                **params,
                'skuv1': v1.generate_sku(copy.deepcopy(params)),
                'skuv2': v2.generate_sku(copy.deepcopy(params))
            })
            assert not row.skuv1.endswith('+')
            assert not row.skuv2.endswith('+')
            if int(row.year) < 2015:
                continue
            rows.append(row)
    sheet.extend(rows)

if __name__ == '__main__':
    asyncio.run(main())


# Copyright (C) 2023 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Any

from headless.types import IClient
from . import extract
from .const import APPLE_MODELS


APPLE_COLORS: dict[str, str] = {
    'Alpine Green'  : 'ALP',
    'Black'         : 'BLA',
    'Black & Slate' : 'BLS',
    'Blue'          : 'BLU',
    'Coral'         : 'COR', # ???
    'Deep Purple'   : 'DEE',
    'Graphite'      : 'GRA',
    'Green'         : 'GRE',
    'Gold'          : 'GOL',
    'Jet Black'     : 'JET',
    'Midnight'      : 'MID',
    'Midnight Green': 'MIG', # ???
    'Pink'          : 'PIN',
    'Red'           : 'RED',
    'Rose Gold'     : 'ROS',
    'PRODUCT(RED)'  : 'PRO',
    'Pacific Blue'  : 'PAC',
    'Purple'        : 'PUR',
    'Sierra Blue'   : 'SIE',
    'Silver'        : 'SIL',
    'Space Black'   : 'SPB',
    'Space Gray'    : 'SPA',
    'Space Grey'    : 'SPA',
    'Sky Blue'      : 'SKY',
    'Starlight'     : 'STA',
    'White'         : 'WHI',
    'White & Silver': 'WHS',
    'Yellow'        : 'YEL',
}
APPLE_COLOR_CODES: dict[str, str] = {v: k for k, v in APPLE_COLORS.items()}
APPLE_COLORS.update({k.lower(): v for k, v in APPLE_COLORS.items()})
GRADES: dict[str, str] = {
    'A+'    : '0',
    'A'     : '1',
    'B+'    : '2',
    'B'     : '3',
    'C+'    : '4',
    'C'     : '5',
    'D'     : '6'
}
GRADES.update({v: k for k, v in GRADES.items()})

def get_color(color: str) -> str:
    return APPLE_COLORS[color]


def generate_sku(params: dict[str, Any]) -> str:
    kind = params.get('kind')
    color_code = params.pop('color_code', None)
    if not params.get('storage') and params.get('memory'):
        params['storage'] = params['memory']
    if params.get('color'):
        color_code = APPLE_COLORS[params['color']]
    match kind:
        case "iPad":
            if params['base'] != 'IPA':
                sku = '{base}{generation}{screen_inch}{color_code}{storage}'.format(color_code=color_code, **params)
            else:
                sku = '{base}{generation}{color_code}{storage}'.format(color_code=color_code, **params)
            sku += 'C' if params.get('cellular') else 'W'
        case "iPhone":
            params.update(APPLE_MODELS[params['model']]) # type: ignore
            params.setdefault('generation', '')
            sku = '{base}{color_code}{storage}'.format(color_code=color_code or '', **params)
        case _:
            raise NotImplementedError
    if params.get('restricted'):
        sku += params['restricted']
    if params.get('grade'):
        grade: str = params['grade']
        params['grade_name'] = GRADES[grade]
        if not grade.isdigit():
            grade = GRADES[grade]
        assert grade.isdigit()
        sku += grade

    if not params.get('device'):
        params['device'] = params['series']
    return sku


def parse_iphones(
    client: IClient[Any, Any]
):
    return extract.parse_iphones(
        client,
        colors=APPLE_COLORS,
        generate_sku=lambda params: '{base}{generation}{color_code}{memory}'.format(**params)
    )


def parse_ipads(
    client: IClient[Any, Any]
):
    return extract.parse_ipads(
        client,
        colors=APPLE_COLORS,
        generate_sku=lambda params: '{base}{generation}{screen_inch}{color_code}{memory}'.format(**params)
    )


# Copyright (C) 2023 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.


APPLE_MODELS: dict[str, dict[str, int | str]] = {
    'iPad'                                  : {'device': 'iPad', 'year': 2010, 'generation': 1, 'base': 'IPA', 'msin': 'ipad:1:fix'},
    'iPad 2'                                : {'device': 'iPad', 'year': 2011, 'generation': 2, 'base': 'IPA', 'msin': 'ipad:2:fix'},
    'iPad (3rd generation)'                 : {'device': 'iPad', 'year': 2012, 'generation': 3, 'base': 'IPA', 'msin': 'ipad:3:fix'},
    'iPad (4th generation)'                 : {'device': 'iPad', 'year': 2012, 'generation': 4, 'base': 'IPA', 'msin': 'ipad:4:fix'},
    'iPad (5th generation)'                 : {'device': 'iPad', 'year': 2017, 'generation': 5, 'base': 'IPA', 'msin': 'ipad:5:fix'},
    'iPad (6th generation)'                 : {'device': 'iPad', 'year': 2018, 'generation': 6, 'base': 'IPA', 'msin': 'ipad:6:fix'},
    'iPad (7th generation)'                 : {'device': 'iPad', 'year': 2019, 'generation': 7, 'base': 'IPA', 'msin': 'ipad:7:fix'},
    'iPad (8th generation)'                 : {'device': 'iPad', 'year': 2020, 'generation': 8, 'base': 'IPA', 'msin': 'ipad:8:fix'},
    'iPad (9th generation)'                 : {'device': 'iPad', 'year': 2021, 'generation': 9, 'base': 'IPA', 'msin': 'ipad:9:fix'},
    'iPad (10th generation)'                : {'device': 'iPad', 'year': 2022, 'generation': 10, 'base': 'IPA', 'msin': 'ipad:10:fix'},
    'iPad Air'                              : {'device': 'iPad Air', 'year': 2013, 'generation': 1, 'base': 'IPAA', 'msin': 'air:1:fix'},
    'iPad Air 2'                            : {'device': 'iPad Air', 'year': 2014, 'generation': 2, 'base': 'IPAA', 'msin': 'air:2:fix'},
    'iPad Air (3rd generation)'             : {'device': 'iPad Air', 'year': 2019, 'generation': 3, 'base': 'IPAA', 'msin': 'air:3:fix'},
    'iPad Air (4th generation)'             : {'device': 'iPad Air', 'year': 2020, 'generation': 4, 'base': 'IPAA', 'msin': 'air:4:fix'},
    'iPad Air (5th generation)'             : {'device': 'iPad Air', 'year': 2022, 'generation': 5, 'base': 'IPAA', 'msin': 'air:5:fix'},
    'iPad Pro (9.7-inch)'                   : {'device': 'iPad Pro', 'year': 2016, 'screen_size': '9.7', 'generation': 1, 'base': 'IPAP', 'msin': 'pro:1:9.7'},
    'iPad Pro (10.5-inch)'                  : {'device': 'iPad Pro', 'year': 2017, 'screen_size': '10.5', 'generation': 1, 'base': 'IPAP', 'msin': 'pro:1:10.5'},
    'iPad Pro (11-inch)'                    : {'device': 'iPad Pro', 'year': 2018, 'screen_size': '11', 'generation': 1, 'base': 'IPAP', 'msin': 'pro:1:11'},
    'iPad Pro (11-inch) (2nd generation)'   : {'device': 'iPad Pro', 'year': 2020, 'screen_size': '11', 'generation': 2, 'base': 'IPAP', 'msin': 'pro:2: 11'},
    'iPad Pro (11-inch) (3rd generation)'   : {'device': 'iPad Pro', 'year': 2021, 'screen_size': '11', 'generation': 3, 'base': 'IPAP', 'msin': 'pro:3:11'},
    'iPad Pro (12.9-inch)'                  : {'device': 'iPad Pro', 'year': 2015, 'screen_size': '12.9', 'generation': 1, 'base': 'IPAP', 'msin': 'pro:1:12.9'},
    'iPad Pro (12.9-inch) (2nd generation)' : {'device': 'iPad Pro', 'year': 2017, 'screen_size': '12.9', 'generation': 2, 'base': 'IPAP', 'msin': 'pro:2:12.9'},
    'iPad Pro (12.9-inch) (3rd generation)' : {'device': 'iPad Pro', 'year': 2018, 'screen_size': '12.9', 'generation': 3, 'base': 'IPAP', 'msin': 'pro:3:12.9'},
    'iPad Pro (12.9-inch) (4th generation)' : {'device': 'iPad Pro', 'year': 2020, 'screen_size': '12.9', 'generation': 4, 'base': 'IPAP', 'msin': 'pro:4:12.9'},
    'iPad Pro (12.9-inch) (5th generation)' : {'device': 'iPad Pro', 'year': 2021, 'screen_size': '12.9', 'generation': 5, 'base': 'IPAP', 'msin': 'pro:5:12.9'},
    'iPad mini'                             : {'device': 'iPad mini', 'year': 2012, 'generation': 1, 'base': 'IPAM', 'msin': 'mini:1:fix'},
    'iPad mini 2'                           : {'device': 'iPad mini', 'year': 2013, 'generation': 2, 'base': 'IPAM', 'msin': 'mini:2:fix'},
    'iPad mini 3'                           : {'device': 'iPad mini', 'year': 2014, 'generation': 3, 'base': 'IPAM', 'msin': 'mini:3:fix'},
    'iPad mini 4'                           : {'device': 'iPad mini', 'year': 2015, 'generation': 4, 'base': 'IPAM', 'msin': 'mini:4:fix'},
    'iPad mini (5th generation)'            : {'device': 'iPad mini', 'year': 2019, 'generation': 5, 'base': 'IPAM', 'msin': 'mini:5:fix'},
    'iPad mini (6th generation)'            : {'device': 'iPad mini', 'year': 2021, 'generation': 6, 'base': 'IPAM', 'msin': 'mini:6:fix'},
    'iPhone'                                : {'year': 2007, 'base': 'IPH1'},
    'iPhone 3G'                             : {'year': 2008, 'base': 'IPH3G'},
    'iPhone 3GS'                            : {'year': 2008, 'base': 'IPH3GS'},
    'iPhone 4'                              : {'year': 2010, 'base': 'IPH4'},
    'iPhone 4S'                             : {'year': 2010, 'base': 'IPH4S'},
    'iPhone 5'                              : {'year': 2012, 'base': 'IPH5'},
    'iPhone 5c'                             : {'year': 2012, 'base': 'IPH5C'},
    'iPhone 5s'                             : {'year': 2012, 'base': 'IPH5S'},
    'iPhone 6'                              : {'year': 2014, 'base': 'IPH6'},
    'iPhone 6 Plus'                         : {'year': 2014, 'base': 'IPH6P'},
    'iPhone 6s'                             : {'year': 2014, 'base': 'IPH6S'},
    'iPhone 6s Plus'                        : {'year': 2014, 'base': 'IPH6SP'},
    'iPhone SE (1st generation)'            : {'year': 2016, 'base': 'IPHSE1', 'generation': 1},
    'iPhone SE (2nd generation)'            : {'year': 2020, 'base': 'IPHSE2', 'generation': 2},
    'iPhone SE (3rd generation)'            : {'year': 2022, 'base': 'IPHSE3', 'generation': 3},
    'iPhone 7'                              : {'year': 2016, 'base': 'IPH7'},
    'iPhone 7 Plus'                         : {'year': 2016, 'base': 'IPH7P'},
    'iPhone 8'                              : {'year': 2017, 'base': 'IPH8'},
    'iPhone 8 Plus'                         : {'year': 2017, 'base': 'IPH8P'},
    'iPhone X'                              : {'year': 2018, 'base': 'IPHX'},
    'iPhone XR'                             : {'year': 2018, 'base': 'IPHXR'},
    'iPhone XS'                             : {'year': 2018, 'base': 'IPHXS'},
    'iPhone XS Max'                         : {'year': 2018, 'base': 'IPHXSM'},
    'iPhone 11'                             : {'year': 2019, 'base': 'IPH11'},
    'iPhone 11 Pro'                         : {'year': 2019, 'base': 'IPH11P'},
    'iPhone 11 Pro Max'                     : {'year': 2019, 'base': 'IPH11PM'},
    'iPhone 12 mini'                        : {'year': 2020, 'base': 'IPH12M'},
    'iPhone 12'                             : {'year': 2020, 'base': 'IPH12'},
    'iPhone 12 Pro'                         : {'year': 2020, 'base': 'IPH12P'},
    'iPhone 12 Pro Max'                     : {'year': 2020, 'base': 'IPH12PM'},
    'iPhone 13 mini'                        : {'year': 2021, 'base': 'IPH13M'},
    'iPhone 13'                             : {'year': 2021, 'base': 'IPH13'},
    'iPhone 13 Pro'                         : {'year': 2021, 'base': 'IPH13P'},
    'iPhone 13 Pro Max'                     : {'year': 2021, 'base': 'IPH13PM'},
    'iPhone 14'                             : {'year': 2022, 'base': 'IPH14'},
    'iPhone 14 Pro'                         : {'year': 2022, 'base': 'IPH14P'},
    'iPhone 14 Plus'                        : {'year': 2022, 'base': 'IPH14PL'},
    'iPhone 14 Pro Max'                     : {'year': 2022, 'base': 'IPH14PM'},
}
APPLE_MODELS.update({
    'iPhone SE 2016': APPLE_MODELS['iPhone SE (1st generation)'],
    'iPhone SE 2020': APPLE_MODELS['iPhone SE (2nd generation)'],
    'iPhone SE 2022': APPLE_MODELS['iPhone SE (3rd generation)'],
})
APPLE_MODELS.update({k.lower(): v for k, v in APPLE_MODELS.items()})


APPLE_IDENTIFIERS: dict[str, dict[str, int | str | bool | None]] = {
    'iPad1,1': {'generation': 1, 'cellular': None},
    'iPad1,2': {'generation': 1, 'cellular': '3G'},
    'iPad2,1': {'generation': 2, 'cellular': None},
    'iPad2,2': {'generation': 2, 'cellular': 'GSM'},
    'iPad2,3': {'generation': 2, 'cellular': 'CDMA'},
    'iPad2,4': {
        'generation': 2,
        'cellular': None
    },
    'iPad3,1': {
        'generation': 3,
        'cellular': None
    },
    'iPad3,2': {
        'generation': 3,
        'cellular': 'CDMA'
    },
    'iPad3,3': {
        'generation': 3,
        'cellular': 'GSM'
    },
    'iPad2,5': {
        'generation': 1,
        'cellular': None
    },
    'iPad2,6': {
        'generation': 1,
        'cellular': 'GSM+LTE'
    },
    'iPad2,7': {
        'generation': 1,
        'cellular': 'CDMA+LTE'
    },
    'iPad3,4': {'generation': 4, 'cellular': None},
    'iPad3,5': {'generation': 4, 'cellular': 'GSM+LTE'},
    'iPad3,6': {'generation': 4, 'cellular': 'CDMA+LTE'},
    'iPad4,1': {'generation': 1, 'cellular': None},
    'iPad4,2': {'generation': 1, 'cellular': 'GSM+CDMA'},
    'iPad4,3': {'generation': 1, 'cellular': 'GSM+CDMA', 'spec': 'CN'},
    'iPad4,4': {'generation': 2, 'cellular': None},
    'iPad4,5': {'generation': 2, 'cellular': 'GSM+CDMA'},
    'iPad4,6': {'generation': 2, 'cellular': 'GSM+CDMA', 'spec': 'CN'},
    'iPad4,7': {'generation': 3, 'cellular': None},
    'iPad4,8': {'generation': 3, 'cellular': 'GSM+CDMA'},
    'iPad4,9': {'generation': 3, 'cellular': 'GSM+CDMA', 'spec': 'CN'},
    'iPad5,1': {'generation': 4, 'cellular': None},
    'iPad5,2': {'generation': 4, 'cellular': "WiFi+Cellular"},
    'iPad5,3': {'generation': 2, 'cellular': None},
    'iPad5,4': {'generation': 2, 'cellular': True},
    'iPad6,3': {'generation': 1, 'cellular': None, 'screen_size': "9.7"},
    'iPad6,4': {'generation': 1, 'cellular': True, 'screen_size': "9.7"},
    'iPad6,7': {'generation': 1, 'cellular': None, 'screen_size': "12.9"},
    'iPad6,8': {'generation': 1, 'cellular': True, 'screen_size': "12.9"},
    'iPad6,11': {'generation': 5, 'cellular': None},
    'iPad6,12': {'generation': 5, 'cellular': True},
    'iPad7,1': {'generation': 2, 'cellular': None, 'screen_size': '12.9'},
    'iPad7,2': {'generation': 2, 'cellular': True, 'screen_size': '12.9'},
    'iPad7,3': {'generation': 1, 'cellular': None, 'screen_size': '10.5'},
    'iPad7,4': {'generation': 1, 'cellular': True, 'screen_size': '10.5'},
    'iPad7,5': {'generation': 6, 'cellular': None},
    'iPad7,6': {'generation': 6, 'cellular': True},
    'iPad7,11': {'generation': 7, 'cellular': None, 'screen_size': '10.2'},
    'iPad7,12': {'generation': 7, 'cellular': True, 'screen_size': '10.2'},
    'iPad8,1': {'generation': 3, 'cellular': None, 'screen_size': '11'},
    'iPad8,2': {'generation': 3, 'cellular': None, 'screen_size': '11'},
    'iPad8,3': {'generation': 3, 'cellular': True, 'screen_size': '11'},
    'iPad8,4': {'generation': 3, 'cellular': True, 'screen_size': '11'},
    'iPad8,5': {'generation': 3, 'cellular': None, 'screen_size': '12.9'},
    'iPad8,6': {'generation': 3, 'cellular': None, 'screen_size': '12.9'},
    'iPad8,7': {'generation': 3, 'cellular': True, 'screen_size': '12.9'},
    'iPad8,8': {'generation': 3, 'cellular': True, 'screen_size': '12.9'},
    'iPad8,9': {'generation': 4, 'cellular': None, 'screen_size': '11'},
    'iPad8,10': {'generation': 4, 'cellular': True, 'screen_size': '11'},
    'iPad8,11': {'generation': 4, 'cellular': None, 'screen_size': '12.9'},
    'iPad8,12': {'generation': 4, 'cellular': True, 'screen_size': '12.9'},
    'iPad11,1': {'generation': 5, 'cellular': None},
    'iPad11,2': {'generation': 5, 'cellular': True},
    'iPad11,3': {'generation': 3, 'cellular': None},
    'iPad11,4': {'generation': 3, 'cellular': True},
    'iPad11,6': {'generation': 8, 'cellular': None},
    'iPad11,7': {'generation': 8, 'cellular': True},
    'iPad12,1': {'generation': 9, 'cellular': None},
    'iPad12,2': {'generation': 9, 'cellular': True},
    'iPad14,1': {'generation': 6, 'cellular': None},
    'iPad14,2': {'generation': 6, 'cellular': True},
    'iPad13,1': {'generation': 4, 'cellular': None},
    'iPad13,2': {'generation': 4, 'cellular': True},
    'iPad13,4': {'generation': 5, 'cellular': None, 'screen_size': '11'},
    'iPad13,5': {'generation': 5, 'cellular': None, 'screen_size': '11'},
    'iPad13,6': {'generation': 5, 'cellular': True, 'screen_size': '11'},
    'iPad13,7': {'generation': 5, 'cellular': True, 'screen_size': '11'},
    'iPad13,8': {'generation': 5, 'cellular': None, 'screen_size': '12.9'},
    'iPad13,9': {'generation': 5, 'cellular': None, 'screen_size': '12.9'},
    'iPad13,10': {'generation': 5, 'cellular': True, 'screen_size': '12.9'},
    'iPad13,11': {'generation': 5, 'cellular': True, 'screen_size': '12.9'},
    'iPad13,16': {'generation': 5, 'cellular': None},
    'iPad13,17': {'generation': 5, 'cellular': True},
    'iPad13,18': {'generation': 10, 'cellular': False},
    'iPad13,19': {'generation': 10, 'cellular': True},
    #'iPad13,18': iPad 10th Gen
    #'iPad13,19': iPad 10th Gen
    #'iPad14,3': iPad Pro 11 inch 4th Gen
    #'iPad14,4': iPad Pro 11 inch 4th Gen
    #'iPad14,5': iPad Pro 12.9 inch 6th Gen
    #'iPad14,6': iPad Pro 12.9 inch 6th Gen
}


DEFAULT_GRADINGS: dict[str, int] = {
    'A+'    : 0,
    'A'     : 1,
    'B+'    : 2,
    'B'     : 3,
    'C+'    : 4,
    'C'     : 5,
    'D'     : 6
}